"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Scope = void 0;
class Scope {
    constructor(scopable = null, enclosingScope = null) {
        this.scopable = scopable;
        this.enclosingScope = enclosingScope;
        this._declaredNames = new Set();
        this._unresolvedNames = new Set();
        this._scheduledTasks = [];
    }
    define(name) {
        if (typeof name === "string") {
            this._declaredNames.add(name);
        }
        else {
            name.forEach((name) => this._declaredNames.add(name));
        }
    }
    find(name) {
        for (let scope = this; scope; scope = scope.enclosingScope) {
            if (scope._declaredNames.has(name)) {
                return true;
            }
        }
        return false;
    }
    resolve(identifier) {
        if (!this.find(identifier.name)) {
            this._unresolvedNames.add(identifier.name);
        }
    }
    hoist(identifier) {
        this._declaredNames.add(identifier.name);
        this._unresolvedNames.delete(identifier.name);
    }
    /**
     * Move unresolved names to its enclosing scope.
     */
    liftUnresolvedNames() {
        if (this.enclosingScope) {
            for (const name of this._unresolvedNames) {
                this.enclosingScope._unresolvedNames.add(name);
            }
        }
    }
    runScheduledTasks() {
        this._scheduledTasks.forEach((task) => task());
    }
    scheduleTask(task) {
        this._scheduledTasks.push(task);
    }
    get unresolvedNames() {
        return Array.from(this._unresolvedNames);
    }
}
exports.Scope = Scope;
